if ($PSEdition -eq "Core" -and $IsWindows -ne $true) {
    Write-Host "This script is only supported on Windows"
    Exit 1
}
elseif ([System.Environment]::OSVersion.Version.Major -lt 10) {
    Write-Host "This script is only supported on Windows 10 or later"
    Exit 2
}

$srv = Get-Service -Name "SCardSvr" -ErrorAction Ignore

if ($srv.Status -ne "Running") {
    Write-Host "Smart Card Service is not running" -ForegroundColor RED
}

$edition = "x86"

if ($env:PROCESSOR_ARCHITECTURE -eq "ARM64") {
    $edition = "arm64"
}

# $env:DRIVER_NAME = "ESTKme-RED"
$redctl = "$PSScriptRoot\bin\redctl-$edition.exe"

if (-not (Test-Path $redctl)) {
    Write-Host "redctl not found at $redctl"
    Exit 4
}

function Get-Version() {
    (& "$redctl" "ver" | Out-String).Trim()
}

function Invoke-Update() {
    & "$redctl" "dfu"
}

function Test-VolumeReady() {
    param(
        [string]$label
    )
    $drive = Get-Volume -FileSystemLabel $label -ErrorAction Ignore
    return $null -ne $drive
}

function Find-Volume() {
    param(
        [string]$label
    )
    $drive = $null
    while ($null -eq $drive) {
        $drive = Get-Volume -FileSystemLabel $label -ErrorAction Ignore
        Start-Sleep -Seconds 1
    }
    return $drive
}

$CUR_VERSION = $null
$OLD_VERSION = $null
$NEW_VERSION = (Get-Content "$PSScriptRoot\VERSION" | Out-String).Trim()
$LABEL = "ESTKme-RED"
$PRODUCT = "ESTKme-RED"
$FIRMWARE = "$PSScriptRoot/estkme-red.fwupd"
$VOLUME_READY = Test-VolumeReady $LABEL

if (-not $VOLUME_READY) {
    $OLD_VERSION = Get-Version
    if (-not $OLD_VERSION) {
        Write-Host "Please connect $PRODUCT to computer"
        Exit 5
    }
    elseif ($OLD_VERSION -eq "0.0.0") {
        Write-Host "REQUIRE USE short-circuit update firmware" -ForegroundColor RED
        Exit 6
    }
    elseif ([Version]$OLD_VERSION -ge [Version]$NEW_VERSION) {
        Write-Host "$PRODUCT is already up-to-date ($OLD_VERSION)"
        Exit 7
    }
}

if ($OLD_VERSION) {
    Write-Host "Old firmware version: $OLD_VERSION"
}
Write-Host "New firmware version: $NEW_VERSION"

if (-not $VOLUME_READY) {
    Write-Host "Enter $PRODUCT DFU mode"

    Invoke-Update
}

$volume = Find-Volume $LABEL

Write-Host "Copying firmware to $LABEL"

Copy-Item $FIRMWARE $volume.Path

Write-Host "Syncing $LABEL"

Write-VolumeCache -Path $volume.Path

Write-Host "Please re-plug $PRODUCT to continue" -ForegroundColor RED

while (-not $CUR_VERSION) {
    Start-Sleep -Seconds 1
    $CUR_VERSION = Get-Version 2> $null
}

if ($CUR_VERSION -eq $NEW_VERSION) {
    Write-Host "Firmware updated successfully ($CUR_VERSION)" -ForegroundColor GREEN
}
else {
    Write-Host "Firmware update failed ($CUR_VERSION)" -ForegroundColor RED
}
